' ########################################################################################
' CSED Editor
' File: CSED_TOOLS.INC
' Contents: Tools database manager
' Copyright (c) 2011 Jos Roca
' All Rights Reserved.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
' EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
' MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ########################################################################################

#COMPILE EXE
#DIM ALL
%UNICODE = 1

$PROGNAME = "CSED External Tools"

' // Include files for external files
#INCLUDE ONCE "windows.inc"
#INCLUDE ONCE "CWindow.inc"
#INCLUDE ONCE "commctrl.inc"
#INCLUDE ONCE "ListBoxCtrl.inc"
#INCLUDE ONCE "EditCtrl.inc"
#INCLUDE ONCE "StatusBarCtrl.inc"
#INCLUDE ONCE "commdlg.inc"
#INCLUDE ONCE "trm.inc"
#INCLUDE ONCE "HtmlHelp.inc"

' // Resource file

' ========================================================================================
' Icon identifiers
' ========================================================================================
%IDI_TOOLS32  = 100
%IDI_TOOLS16  = 101
%IDI_FOLDER16 = 102
%IDI_FOLDER24 = 103
' ========================================================================================

#RESOURCE MANIFEST, 1, "CSED_TOOLS.xml"
#RESOURCE ICON,   100, "32_TOOLS.ico"
#RESOURCE ICON,   101, "16_TOOLS.ico"
#RESOURCE ICON,   102, "16_FOLDR.ico"
#RESOURCE ICON,   103, "24_FOLDR.ico"

' ========================================================================================
' Control identifiers
' ========================================================================================
ENUM CSED_TOOLS_ENUM SINGULAR
   IDC_STATUSBAR = 1001
   IDC_LBITEMS
   IDC_NAME
   IDC_PATH
   IDIB_PATH
   IDC_PARAMETERS
   IDC_MENU
   IDB_EXIT
   IDB_SAVE
   IDB_EXECUTE
   IDB_DELETE
   IDB_NEW
END ENUM
' ========================================================================================

' ========================================================================================
' Record structure
' ========================================================================================
TYPE CSED_TOOLSTYPE
   strName   AS STRING * 64
   strPath   AS STRING * 256
   strParams AS STRING * 1023
   strMenu   AS STRING * 1
END TYPE
' ========================================================================================

' ========================================================================================
' Global handle
' ========================================================================================
GLOBAL hToolsFile AS LONG
' ========================================================================================

' ========================================================================================
' Creates the Tools database
' ========================================================================================
FUNCTION CreateToolsDatabase (BYVAL hwnd AS DWORD, BYVAL strDbPath AS STRING) AS LONG

   LOCAL strFileDef AS STRING
   LOCAL strKeys AS STRING
   LOCAL strSegment AS STRING * 25

   strSegment = "Tool Name"

   strFileDef = MKBYT$(1) & _     ' // Page Size
                MKBYT$(1) & _     ' // Compress Record
                MKBYT$(1)         ' // Number of Key Segments

   strKeys =    strSegment  & _   ' // Description
                MKBYT$(1)   & _   ' // Segment number
                MKWRD$(1)   & _   ' // Segment Offset
                MKBYT$(64)  & _   ' // Segment Size
                MKBYT$(2)         ' // Flags (No duplicates)

   strFileDef += strKeys

   FUNCTION = trm_Create(strDbPath, strFileDef, %FALSE)

END FUNCTION
' ========================================================================================

' ========================================================================================
' Save the record
' ========================================================================================
FUNCTION SaveToolsRecord (BYVAL hwnd AS DWORD) AS LONG

   LOCAL Result AS LONG
   LOCAL RecordStr AS STRING
   LOCAL strText AS STRING
   LOCAL idx AS LONG
   LOCAL tTools AS CSED_TOOLSTYPE

   strText = Edit_GetText(GetDlgItem(hwnd, %IDC_NAME))
   IF ISNULL(strText) THEN
      MessageBox hwnd, "The name must be filled", "CSED_Tools", %MB_ICONERROR OR %MB_OK OR %MB_APPLMODAL
      SetFocus GetDlgItem(hwnd, %IDC_NAME)
      EXIT FUNCTION
   END IF
   tTools.strName = strText
   strText = Edit_GetText(GetDlgItem(hwnd, %IDC_PATH))
   IF ISNULL(strText) THEN
      MessageBox hwnd, "The path must be filled", "CSED_Tools", %MB_ICONERROR OR %MB_OK OR %MB_APPLMODAL
      SetFocus GetDlgItem(hwnd, %IDC_PATH)
      EXIT FUNCTION
   END IF
   IF ISFALSE ISFILE(strText) THEN
      MessageBox hwnd, "Invalid path " & strText, "CSED_Tools", %MB_ICONERROR OR %MB_OK OR %MB_APPLMODAL
      SetFocus GetDlgItem(hwnd, %IDC_PATH)
      EXIT FUNCTION
   END IF
   tTools.strPath = strText
   strText = Edit_GetText(GetDlgItem(hwnd, %IDC_PARAMETERS))
   tTools.strParams = strText
   IF IsDlgButtonChecked(hwnd, %IDC_MENU) = %BST_CHECKED THEN tTools.strMenu = "1" ELSE tTools.strMenu = ""

   RecordStr = tTools
   Result = trm_Insert(hToolsFile, RecordStr)
   IF Result = 5 THEN   ' // Duplicate key
      Result = trm_Delete(hToolsFile)
      Result = trm_Insert(hToolsFile, BYCOPY RecordStr)
      Statusbar_SetText(GetDlgItem(hwnd, %IDC_STATUSBAR), 0, "Record updated")
      idx = ListBox_FindStringExact(GetDlgItem(hwnd, %IDC_LBITEMS), 0, TRIM$(tTools.strName))
      IF idx <> %LB_ERR THEN ListBox_SetCursel(GetDlgItem(hwnd, %IDC_LBITEMS), idx)
   ELSEIF Result = 0 THEN
      idx = Listbox_AddString(GetDlgItem(hwnd, %IDC_LBITEMS), TRIM$(tTools.strName))
      ListBox_SetCursel(GetDlgItem(hwnd, %IDC_LBITEMS), idx)
      Statusbar_SetText(GetDlgItem(hwnd, %IDC_STATUSBAR), 0, "Record added")
      RetrieveToolsRecord hwnd
   ELSE
      MessageBox hwnd, "Error" & STR$(Result) & " saving the record", "CSED_Tools", %MB_ICONERROR OR %MB_OK OR %MB_APPLMODAL
   END IF

   FUNCTION = Result

END FUNCTION
' ========================================================================================

' ========================================================================================
' Retrieve the record
' ========================================================================================
FUNCTION RetrieveToolsRecord (BYVAL hwnd AS DWORD) AS LONG

   LOCAL RecordStr AS STRING
   LOCAL strName AS STRING
   LOCAL Index AS LONG
   LOCAL tTools AS CSED_TOOLSTYPE

   IF ISFALSE hToolsFile THEN EXIT FUNCTION
   Index = Listbox_GetCurSel(GetDlgItem(hwnd, %IDC_LBITEMS))
   IF Index = %LB_ERR THEN EXIT FUNCTION
   strName = ListBox_GetText(GetDlgItem(hwnd, %IDC_LBITEMS), Index)
   IF LEN(strName) > 64 THEN strName = LEFT$(strName, 64)
   RecordStr = trm_GetEqualOrGreater(hToolsFile, 1, strName)
   LSET tTools = RecordStr
   IF UCASE$(TRIM$(LEFT$(RecordStr, 64))) = UCASE$(strName) THEN
      SetWindowText GetDlgItem(hwnd, %IDC_NAME), TRIM$(tTools.strName)
      SetWindowText GetDlgItem(hwnd, %IDC_PATH), TRIM$(tTools.strPath)
      SetWindowText GetDlgItem(hwnd, %IDC_PARAMETERS), TRIM$(tTools.strParams)
      CheckDlgButton hwnd, %IDC_MENU, IIF&(VAL(tTools.strMenu) = 1, %BST_CHECKED, %BST_UNCHECKED)
      Statusbar_SetText(GetDlgItem(hwnd, %IDC_STATUSBAR), 0, TRIM$(tTools.strPath))
   END IF

END FUNCTION
' ========================================================================================

' ========================================================================================
' Delete the record
' ========================================================================================
FUNCTION DeleteToolsRecord (BYVAL hwnd AS DWORD) AS LONG

   LOCAL RecordStr AS STRING
   LOCAL strName AS STRING
   LOCAL Index AS LONG
   LOCAL Result AS LONG
   LOCAL tTools AS CSED_TOOLSTYPE

   IF ISFALSE hToolsFile THEN EXIT FUNCTION
   Index = Listbox_GetCurSel(GetDlgItem(hwnd, %IDC_LBITEMS))
   strName = ListBox_GetText(GetDlgItem(hwnd, %IDC_LBITEMS), Index)
   IF LEN(strName) > 64 THEN strName = LEFT$(strName, 64)
   RecordStr = trm_GetEqualOrGreater(hToolsFile, 1, strName)
   LSET tTools = RecordStr
   IF UCASE$(TRIM$(LEFT$(RecordStr, 64))) = UCASE$(strName) THEN
      Result = trm_Delete(hToolsFile)
      IF Result = 0 THEN
         Statusbar_SetText(GetDlgItem(hwnd, %IDC_STATUSBAR), 0, "Record deleted")
         Listbox_DeleteString(GetDlgItem(hwnd, %IDC_LBITEMS), Index)
         ListBox_SetCursel(GetDlgItem(hwnd, %IDC_LBITEMS), 0)
         RetrieveToolsRecord(hwnd)
      ELSE
         MessageBox hwnd, "Error" & STR$(Result) & " deleting the record", "CSED_Tools", %MB_ICONERROR OR %MB_OK OR %MB_APPLMODAL
      END IF
   END IF

END FUNCTION
' ========================================================================================

' ========================================================================================
' Execute the tool and close this application
' ========================================================================================
FUNCTION ExecuteTool (BYVAL hwnd AS DWORD) AS LONG

   LOCAL PID AS DWORD
   LOCAL Index AS LONG
   LOCAL RecordStr AS STRING
   LOCAL strName AS STRING
   LOCAL tTools AS CSED_TOOLSTYPE

   IF ISFALSE hToolsFile THEN EXIT FUNCTION
   Index = Listbox_GetCurSel(GetDlgItem(hwnd, %IDC_LBITEMS))
   strName = ListBox_GetText(GetDlgItem(hwnd, %IDC_LBITEMS), Index)
   IF LEN(strName) > 64 THEN strName = LEFT$(strName, 64)
   RecordStr = trm_GetEqualOrGreater(hToolsFile, 1, strName)
   LSET tTools = RecordStr
   IF UCASE$(TRIM$(LEFT$(RecordStr, 64))) = UCASE$(strName) THEN
      IF UCASE$(RIGHT$(TRIM$(tTools.strPath), 4)) = ".CHM" THEN
         HtmlHelp( 0, BYCOPY TRIM$(tTools.strPath), 0, 0)
      ELSEIF UCASE$(RIGHT$(TRIM$(tTools.strPath), 4)) = ".EXE" THEN
         PID = SHELL(TRIM$(tTools.strPath) & " " & TRIM$(tTools.strParams), 1)
         IF PID THEN PostMessage(hwnd, %WM_CLOSE, 0, 0)
      ELSE
         ShellExecute(%NULL, "open", BYCOPY TRIM$(tTools.strPath), BYCOPY TRIM$(tTools.strParams), "", %SW_SHOWNORMAL)
      END IF
   END IF

END FUNCTION
' ========================================================================================

' ========================================================================================
' Fill the listbox
' ========================================================================================
FUNCTION FillListBox (BYVAL hwnd AS DWORD) AS LONG

   LOCAL RecordStr AS STRING
   LOCAL nCount AS LONG
   LOCAL tTools AS CSED_TOOLSTYPE

   RecordStr = trm_GetFirst(hToolsFile, 1)
   WHILE LEN(RecordStr)
      INCR nCount
      AfxDoEvents hwnd
      LSET tTools = RecordStr
      Listbox_AddString GetDlgItem(hwnd, %IDC_LBITEMS), TRIM$(tTools.strName)
      RecordStr = trm_GetNext(hToolsFile)
   WEND

   FUNCTION = nCount

END FUNCTION
' ========================================================================================

' ========================================================================================
' Common dialog to search the tool path
' ========================================================================================
FUNCTION CSED_GetToolPath (BYVAL hwndParent AS DWORD) AS STRING

   LOCAL strFilter AS STRING
   LOCAL dwStyle AS DWORD
#IF %DEF(%UNICODE)
   LOCAL strFileSpec AS WSTRING
   STATIC strInitialDir AS WSTRING
#ELSE
   LOCAL strFileSpec AS STRING
   STATIC strInitialDir AS STRING
#ENDIF

   IF ISNULL(strInitialDir) THEN strInitialDir = CURDIR$
   strFilter  = "Executable files (*.EXE)|*.EXE|"
   strFilter += "Help files (*.CHM)|*.CHM|"
   strFilter += "PDF files (*.PDF)|*.PDF|"
   strFilter += "All files (*.*)|*.*"
   strFileSpec = "*.EXE;*.CHM;*.PDF"
   dwStyle = %OFN_EXPLORER OR %OFN_FILEMUSTEXIST OR %OFN_HIDEREADONLY
   IF ISTRUE OpenFileDialog(hwndParent, "", strFileSpec, strInitialDir, strFilter, "EXE", dwStyle) THEN FUNCTION = strFileSpec
   strInitialDir = PATHSCAN$(PATH, strFileSpec)

END FUNCTION
' ========================================================================================

' ########################################################################################
' Main
' ########################################################################################
FUNCTION WinMain (BYVAL hInstance AS DWORD, BYVAL hPrevInstance AS DWORD, BYVAL lpszCmdLine AS ASCIIZ PTR, BYVAL nCmdShow AS LONG) AS LONG

   ' // Mutex to avoid multiple instances running
   LOCAL hWindow AS DWORD
   LOCAL hMutex AS DWORD
#IF %DEF(%UNICODE)
   LOCAL szMutex AS WSTRINGZ * 260
#ELSE
   LOCAL szMutex AS ASCIIZ * 260
#ENDIF
   szMutex = "CSED_TOOLS_MUTEX"
   hMutex  = CreateMutex(BYVAL %NULL, %FALSE, szMutex)
   IF ISTRUE hMutex THEN
      IF GetLastError = %ERROR_ALREADY_EXISTS THEN
         CloseHandle hMutex
         hWindow = FindWindow(BYVAL %NULL, $PROGNAME)
         IF hWindow THEN SetForegroundWindow hWindow
         FUNCTION = -1
         EXIT FUNCTION
      END IF
   END IF

   ' // Create an instance of the class
   LOCAL pWindow AS IWindow
   pWindow = CLASS "CWindow"
   IF ISNOTHING(pWindow) THEN EXIT FUNCTION

   ' // Create the main window
   LOCAL hwnd AS DWORD
   hwnd = pWindow.CreateWindow(%NULL, $PROGNAME, 0, 0, 0, 0, _
      %WS_OVERLAPPED OR %WS_VISIBLE OR %WS_MINIMIZEBOX OR %WS_SYSMENU, _
      %WS_EX_WINDOWEDGE, CODEPTR(WndProc))
   pWindow.SetClientSize 602, 242
   ' // Center the window
   pWindow.CenterWindow hwnd

   ' // Set the icons
   pWindow.BigIcon = LoadIcon(hInstance, BYVAL %IDI_TOOLS32)
   pWindow.SmallIcon = LoadIcon(hInstance, BYVAL %IDI_TOOLS16)

   ' // Add the controls
   LOCAL hCtl AS DWORD

   pWindow.AddStatusBar(hwnd, %IDC_STATUSBAR, "", 0, 0, 0, 0, %WS_CHILD OR %WS_VISIBLE OR %WS_CLIPCHILDREN OR %WS_CLIPSIBLINGS OR %CCS_BOTTOM)
   ' // Update the size of the statusbar
   SendMessage GetDlgItem(hwnd, %IDC_STATUSBAR), %WM_SIZE, 0, 0

   pWindow.AddListBox(hwnd, %IDC_LBITEMS, "", 0, 0, 0, 0, %WS_CHILD OR %WS_VISIBLE OR %WS_HSCROLL OR %WS_VSCROLL OR _
      %WS_BORDER OR %WS_TABSTOP OR %LBS_STANDARD OR %LBS_HASSTRINGS OR %LBS_SORT OR %LBS_NOTIFY OR %LBS_NOINTEGRALHEIGHT, -1)
   ' // Adjust the height of the control so that the integral height
   ' // is based on the new font rather than the default SYSTEM_FONT
   pWindow.SetWindowPos GetDlgItem(hwnd, %IDC_LBITEMS), %NULL, 15, 15, 201, 188, %SWP_NOZORDER
   SendMessage GetDlgItem(hwnd, %IDC_LBITEMS), %LB_SETHORIZONTALEXTENT, 600, 0

   pWindow.AddLabel(hwnd, -1, "Name", 230, 15, 358, 15, -1, -1)

   hCtl = pWindow.AddTextBox(hwnd, %IDC_NAME, "", 230, 30, 358, 21, -1, -1)
   ' // Set the text limit
   SendMessage hCtl, %EM_SETLIMITTEXT, 64, 0

   pWindow.AddLabel(hwnd, -1, "Path", 230, 55, 328, 15, -1, -1)

   hCtl = pWindow.AddTextBox(hwnd, %IDC_PATH, "", 230, 70, 328, 21, -1, -1)
   SendMessage hCtl, %EM_SETLIMITTEXT, 256, 0

   IF pWindow.DPI => 144 THEN
      pWindow.AddIconButton(hwnd, %IDIB_PATH, "#" & FORMAT$(%IDI_FOLDER24), 562, 70, 26, 21, -1, -1)
   ELSE
      pWindow.AddIconButton(hwnd, %IDIB_PATH, "#" & FORMAT$(%IDI_FOLDER16), 562, 70, 26, 21, -1, -1)
   END IF
   pWindow.AddLabel(hwnd, -1, "Parameters", 230, 95, 358, 15, -1, -1)

   hCtl = pWindow.AddTextBox(hwnd, %IDC_PARAMETERS, "", 230, 110, 358, 21, -1, -1)
   SendMessage hCtl, %EM_SETLIMITTEXT, 1023, 0

   hCtl = pWindow.AddCheckBox(hwnd, %IDC_MENU, "Display this item in the editor's menu", 230, 145, 250, 16, -1)

   pWindow.AddButton(hwnd, %IDB_SAVE, "&Save", 230, 177, 65, 26, -1, -1)
   pWindow.AddButton(hwnd, %IDB_NEW, "&New", 303, 177, 65, 26, -1, -1)
   pWindow.AddButton(hwnd, %IDB_DELETE, "&Delete", 376, 177, 65, 26, -1, -1)
   pWindow.AddButton(hwnd, %IDB_EXECUTE, "&Execute", 449, 177, 65, 26, -1, -1)
   pWindow.AddButton(hwnd, %IDB_EXIT, "E&xit", 522, 177, 65, 26, -1, -1)

   LOCAL strDbPath AS STRING
   strDbPath = EXE.Path$ & "CSED_TOOLS.TRM"
   IF ISFALSE ISFILE(strDbPath) THEN
      CreateToolsDatabase(hwnd, strDbPath)
   END IF

   hToolsFile = trm_Open(strDbPath, %TRUE)
   IF hToolsFile < 1 THEN
      MessageBox hwnd, "Error opening " & strDbPath, "CSED_Tools", %MB_ICONERROR OR %MB_OK OR %MB_APPLMODAL
      EXIT FUNCTION
   END IF

   IF ISFALSE FillListBox(hwnd) THEN
      SetFocus GetDlgItem(hwnd, %IDC_NAME)
   ELSE
      SetFocus GetDlgItem(hwnd, %IDC_LBITEMS)
      ListBox_SetCursel GetDlgItem(hwnd, %IDC_LBITEMS), 0
      RetrieveToolsRecord hwnd
   END IF

   ' // Default message pump
   pWindow.DoEvents(nCmdShow)

   ' // Close the Tsunami database file
   IF hToolsFile THEN trm_Close(hToolsFile)

END FUNCTION
' ========================================================================================

' ========================================================================================
' Processes messages for the main window.
' ========================================================================================
FUNCTION WndProc (BYVAL hwnd AS DWORD, BYVAL uMsg AS DWORD, BYVAL wParam AS DWORD, BYVAL lParam AS LONG) AS LONG

   LOCAL  strPath AS STRING
   STATIC hFocus  AS DWORD

   SELECT CASE uMsg

      CASE %WM_ACTIVATEAPP
         IF ISFALSE wParam THEN hFocus = GetFocus

      CASE %WM_SETFOCUS
         IF hFocus THEN
            PostMessage hwnd, %WM_USER + 999, hFocus, 0
            hFocus = 0
         ELSE
            ' // Set the keyboard focus to the first control that is
            ' // visible, not disabled, and has the WS_TABSTOP style
            SetFocus GetNextDlgTabItem(hwnd, %NULL, %FALSE)
         END IF

      CASE %WM_USER + 999
         IF ISTRUE wParam THEN SetFocus wParam

      CASE %WM_SIZE
         IF wParam <> %SIZE_MINIMIZED THEN
            ' // Update the size and position of aligned controls
            SendMessage GetDlgItem(hwnd, %IDC_STATUSBAR), %WM_SIZE, wParam, lParam
         END IF

      CASE %WM_SYSCOLORCHANGE
         ' // Forward this message to common controls so that they will
         ' // be properly updated when the user changes the color settings.
         SendMessage GetDlgItem(hwnd, %IDC_STATUSBAR), %WM_SYSCOLORCHANGE, wParam, lParam

      CASE %WM_COMMAND

         SELECT CASE LO(WORD, wParam)

            CASE %IDCANCEL
               IF HI(WORD, wParam) = %BN_CLICKED THEN
                  SendMessage hwnd, %WM_CLOSE, 0, 0
                  EXIT FUNCTION
               END IF

            CASE %IDC_LBITEMS
               SELECT CASE HI(WORD, wParam)
                  CASE %LBN_DBLCLK
                     ExecuteTool(hwnd)
                     EXIT FUNCTION
                  CASE %LBN_SELCHANGE
                     RetrieveToolsRecord hwnd
                     EXIT FUNCTION
               END SELECT

            CASE %IDIB_PATH
               IF HI(WORD, wParam) = %BN_CLICKED THEN
                  strPath = CSED_GetToolPath(hwnd)
                  IF LEN(strPath) THEN SetWindowText GetDlgItem(hwnd, %IDC_PATH), BYCOPY strPath
                  SetFocus GetDlgItem(hwnd, %IDC_PATH)
                  EXIT FUNCTION
               END IF

            CASE %IDB_SAVE, %IDOK
               IF HI(WORD, wParam) = %BN_CLICKED THEN
                  SaveToolsRecord(hwnd)
                  EXIT FUNCTION
               END IF

            CASE %IDB_NEW
               IF HI(WORD, wParam) = %BN_CLICKED THEN
                  SetWindowText GetDlgItem(hwnd, %IDC_NAME), ""
                  SetWindowText GetDlgItem(hwnd, %IDC_PATH), ""
                  SetWindowText GetDlgItem(hwnd, %IDC_PARAMETERS), ""
                  CheckDlgButton hwnd, %IDC_MENU, %BST_UNCHECKED
                  Statusbar_SetText(GetDlgItem(hwnd, %IDC_STATUSBAR), 0, "")
                  ListBox_SetCursel GetDlgItem(hwnd, %IDC_LBITEMS), -1
                  SetFocus GetDlgItem(hwnd, %IDC_NAME)
                  EXIT FUNCTION
               END IF

            CASE %IDB_DELETE
               IF HI(WORD, wParam) = %BN_CLICKED THEN
                  DeleteToolsRecord(hwnd)
                  Statusbar_SetText(GetDlgItem(hwnd, %IDC_STATUSBAR), 0, "")
                  SetFocus GetDlgItem(hwnd, %IDC_LBITEMS)
                  EXIT FUNCTION
               END IF

            CASE %IDB_EXECUTE
               IF HI(WORD, wParam) = %BN_CLICKED THEN
                  ExecuteTool(hwnd)
                  EXIT FUNCTION
               END IF

            CASE %IDB_EXIT
               IF HI(WORD, wParam) = %BN_CLICKED THEN
                  SendMessage hwnd, %WM_CLOSE, 0, 0
                  EXIT FUNCTION
               END IF

         END SELECT

      CASE %WM_DESTROY
         ' // Destroy background images
         DeleteObject SendMessage(GetDlgItem(hwnd, %IDIB_PATH), %BM_SETIMAGE, %IMAGE_ICON, %NULL)
         PostQuitMessage 0
         EXIT FUNCTION

   END SELECT

   FUNCTION = DefWindowProc(hwnd, uMsg, wParam, lParam)

END FUNCTION
' ========================================================================================
