' ########################################################################################
' CSED Editor
' File: CSED_FILE.INC
' Contents: File related functions and procedures
' Copyright (c) 2011 Jos Roca
' All Rights Reserved.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
' EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
' MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ########################################################################################

' ========================================================================================
' Retrieves the date and time that a file was last modified.
' ========================================================================================
#If %Def(%UNICODE)
Function CSED_GetFileTime (ByVal strFileSpec As WString) As Dword
#Else
Function CSED_GetFileTime (ByVal strFileSpec As String) As Dword
#EndIf

   Local fd      As WIN32_FIND_DATA
   Local ft      As SYSTEMTIME
   Local hFile   As Dword
   Local strTime As String

   hFile = FindFirstFile(ByCopy strFileSpec, fd)
   If hFile = %INVALID_HANDLE_VALUE Then Exit Function
   FindClose hFile

   ' // Convert the file time into a compatible system time
   FileTimeToSystemTime fd.ftLastWriteTime, ft
   strTime = Format$(ft.wHour) & Format$(ft.wMinute) & Format$(ft.wSecond)
   Function = Val(strTime)

End Function
' ========================================================================================

' ========================================================================================
' Search for compiler paths
' ========================================================================================
#If %Def(%UNICODE)
Function CSED_SearchForExePath (ByVal hwnd As Dword) As WString
#Else
Function CSED_SearchForExePath (ByVal hwnd As Dword) As String
#EndIf

   Local dwStyle       As Dword
#If %Def(%UNICODE)
   Local strFileSpec   As WString
   Local strFilter     As WString
   Local strInitialDir As WString
#Else
   Local strFileSpec   As String
   Local strFilter     As String
   Local strInitialDir As String
#EndIf

   strInitialDir = CurDir$
   strFilter = "EXE files (*.EXE)|*.EXE|"
   strFilter += "All Files (*.*)|*.*"
   strFileSpec = "*.EXE"
   dwStyle = %OFN_Explorer Or %OFN_FileMustExist Or %OFN_HideReadOnly
   If IsTrue AfxOpenFileDialog(hwnd, "", strFileSpec, strInitialDir, strFilter, "EXE", dwStyle) Then Function = strFileSpec

End Function
' ========================================================================================

' ========================================================================================
' Search for help paths
' ========================================================================================
#If %Def(%UNICODE)
Function CSED_SearchForHelpFilePath (ByVal hwnd As Dword) As WString
#Else
Function CSED_SearchForHelpFilePath (ByVal hwnd As Dword) As String
#EndIf

   Local dwStyle       As Dword
#If %Def(%UNICODE)
   Local strFileSpec   As WString
   Local strFilter     As WString
   Local strInitialDir As WString
#Else
   Local strFileSpec   As String
   Local strFilter     As String
   Local strInitialDir As String
#EndIf

   strInitialDir = CurDir$
   strFilter = "Help files (*.HLP;*:CHM)|*.HLP|*.CHM|"
   strFilter += "All Files (*.*)|*.*"
   strFileSpec = "*.HLP;*.CHM"
   dwStyle = %OFN_Explorer Or %OFN_FileMustExist Or %OFN_HideReadOnly
   If IsTrue AfxOpenFileDialog(hwnd, "", strFileSpec, strInitialDir, strFilter, "CHM", dwStyle) Then Function = strFileSpec

End Function
' ========================================================================================

' ========================================================================================
' Search for include files
' ========================================================================================
#If %Def(%UNICODE)
Function CSED_SearchForIncFilesPath (ByVal hwnd As Dword) As WString
#Else
Function CSED_SearchForIncFilesPath (ByVal hwnd As Dword) As String
#EndIf

   Local dwStyle       As Dword
#If %Def(%UNICODE)
   Local strFileSpec   As WString
   Local strFilter     As WString
   Local strInitialDir As WString
#Else
   Local strFileSpec   As String
   Local strFilter     As String
   Local strInitialDir As String
#EndIf

   strInitialDir = CurDir$
   strFilter = "INC files (*.INC)|*.INC|"
   strFilter += "BAS files (*.BAS)|*.BAS|"
   strFileSpec = "*.HLP;*.CHM"
   dwStyle = %OFN_Explorer Or %OFN_FileMustExist Or %OFN_HideReadOnly
   If IsTrue AfxOpenFileDialog(hwnd, "", strFileSpec, strInitialDir, strFilter, "INC", dwStyle) Then Function = strFileSpec

End Function
' ========================================================================================

' ========================================================================================
' Browse for folder dialog procedure
' ========================================================================================
Function CSED_BrowseForFolderProc (ByVal hwnd As Dword, ByVal wMsg As Dword, ByVal wParam As Dword, ByVal lParam As Long) As Long

#If %Def(%UNICODE)
   Local szBuffer As WStringZ * %Max_Path
#Else
   Local szBuffer As AsciiZ * %Max_Path
#EndIf

   If wMsg = %BFFM_INITIALIZED Then
      SendMessage hWnd, %BFFM_SETSELECTION, %TRUE, lParam
   ElseIf wMsg = %BFFM_SELCHANGED Then
      SHGetPathFromIDList ByVal wParam, szBuffer
      If IsFalse wParam Or _                            ' No id number
         IsFalse Len(szBuffer) Or _                     ' No name
         IsFalse (GetAttr(szBuffer) And %SubDir) Or _   ' Not a real subdir
         Mid$(szBuffer, 2, 1) <> ":" Then               ' Not a local or mapped drive
           SendMessage hwnd, %BFFM_ENABLEOK, %FALSE, %FALSE
           Beep
       ElseIf (GetAttr(szBuffer) And %System) And Right$(szBuffer,2) <> ":\" Then ' exclude system folders, allow root directories
         SendMessage hwnd, %BFFM_ENABLEOK, %FALSE, %FALSE
         Beep
       End If
   End If

End Function
' ========================================================================================

' ========================================================================================
' Browse for folder dialog
' ========================================================================================
#If %Def(%UNICODE)
Function CSED_BrowseForFolder (hwnd As Dword, strTitle As WString, startFolder As WString) As String
#Else
Function CSED_BrowseForFolder (hwnd As Dword, strTitle As String, startFolder As String) As String
#EndIf

#If %Def(%UNICODE)
   Local szBuffer As WStringZ * %Max_Path
#Else
   Local szBuffer As AsciiZ * %Max_Path
#EndIf
   Local bi       As BROWSEINFO
   Local lpIDList As Long

   bi.hWndOwner    = hWnd
   bi.lpszTitle    = StrPtr(strTitle)
   bi.ulFlags      = %BIF_ReturnOnlyFSDirs Or %BIF_DontGoBelowDomain Or %BIF_UseNewUI Or %BIF_ReturnFSAncestors
   bi.lpfnCallback = CodePtr(CSED_BrowseForFolderProc)
   bi.lParam       = StrPtr(StartFolder)
   lpIDList        = SHBrowseForFolder(bi)

   If IsTrue lpIDList And SHGetPathFromIDList(ByVal lpIDList, szBuffer) Then
      CoTaskMemFree lpIDList
      Function = szBuffer
   End If

End Function
' ========================================================================================

' ========================================================================================
' Open selected files in MDI windows
' ========================================================================================
Function CSED_OpenFileDialog (ByVal hwnd As Dword) As Dword

   Local i               As Long
   Local nCount          As Long
   Local hMdi            As Dword
   Local dwStyle         As Dword
#If %Def(%UNICODE)
   Local strInitialDir   As WString
   Local strFileSpec     As WString
   Local strDefExtension As WString
   Local strFilter       As WString
   Local strPath         As WString
   Local strFile         As WString
#Else
   Local strInitialDir   As String
   Local strFileSpec     As String
   Local strDefExtension As String
   Local strFilter       As String
   Local strPath         As String
   Local strFile         As String
#EndIf

   strInitialDir = CurDir$
   If IsFile(ProjectFile) Then strInitialDir = PathName$(Path,ProjectFile)
   strFileSpec = "*.BAS;*.INC"
   strDefExtension = "BAS"
   strFilter  = "PB Code Files (*.BAS)|*.BAS|"
   strFilter += "PB Include Files (*.INC)|*.INC|"
   strFilter += "PB Template Files (*.PBTPL)|*.PBTPL|"
   strFilter += "Resource Files (*.RC;*.RC2;.DLG)|*.RC;*.RC2;*.DLG|"
   strFilter += "Text Files (*.TXT)|*.TXT|"
   strFilter += "XML Files (*.XML;*.XSL;*.XSD)|*.XML;*.XSL;*.XSD|"
   strFilter += "HTML Files (*.HTML;*.HTM;*.SHTML)|*.HTML;*.HTM;*.SHTML|"
   strFilter += "C/CPP Files (*.C;*.CC;*.CPP;*.CXX;*.H;*.HH;*.HPP;*.HXX)|*.C;*.CC;*.CPP;*.CXX;*.H;*.HH;*.HPP;*.HXX|"
   strFilter += "Java Files (*.JAVA;*.JAD)|*.JAVA;*.JAD|"
   strFilter += "Java Script Files (*.JS)|*.JS|"
   strFilter += "IDL Files (*.IDL;*.ODL)|*.IDL;*.ODL|"
   strFilter += "All Files (*.*)|*.*"
   dwStyle = %OFN_Explorer Or %OFN_FileMustExist Or %OFN_AllowMultiSelect

   If AfxOpenFileDialog(hwnd, "", strFileSpec, strInitialDir, strFilter, strDefExtension, dwStyle) Then
      strFileSpec = RTrim$(strFileSpec, Chr$(0))
      nCount = ParseCount(strFileSpec, Chr$(0))
      If nCount = 1 Then
         hMdi = CSED_OpenFile(strFileSpec)
         ' // If the user has checked the open read-only box don't allow editing
         If (dwStyle And %OFN_ReadOnly) = %OFN_ReadOnly Then
            SCI_SetReadOnly(pSed.hEdit, %TRUE)
         End If
      Else
         strPath = Parse$(strFileSpec, Chr$(0), 1)
         If Right$(strPath, 1) <> "\" Then strPath = strPath & "\"
         For i = 2 To nCount
            strFile = Parse$(strFileSpec, Chr$(0), i)
            If Len(strFile) Then
               hMdi = CSED_OpenFile(strPath & strFile)
               ' // If the user has checked the open read-only box don't allow editing
               ' // Note: Doesn't work with multiselection: the flag is cleared.
'               IF (dwStyle AND %OFN_READONLY) = %OFN_READONLY THEN
'                  IF hMdi THEN SCI_SetReadOnly(pSed.hEdit, %TRUE)
'               END IF
            End If
         Next
      End If
   End If

   Function = hMdi

End Function
' ========================================================================================

' ========================================================================================
' Open file function
' ========================================================================================
#If %Def(%UNICODE)
Function CSED_OpenFile (ByVal strFileName As WString) As Dword
#Else
Function CSED_OpenFile (ByVal strFileName As String) As Dword
#EndIf

   Local hMdi      As Dword     ' // Handle of the MDI window
   Local nFile     As Long      ' // File handle
   Local fModify   As Long      ' // Modify flag - indicates if the file has been mdified and needs saving
   Local fTime     As Dword     ' // File time
   Local bValidExt As Long      ' // Valid extension
   Local nTab      As Long      ' // Tab index
   Local p         As Long      ' // Position
   Local r         As Long      ' // Result code
   Local buffer    As String    ' // Buffer (ansi)
#If %Def(%UNICODE)
   Local strExt    As WString   ' // File extension
   Local strPath   As WString   ' // Path and name of the file
#Else
   Local strExt    As String    ' // File extension
   Local strPath   As String    ' // Path and name of the file
#EndIf

   If Len(strFileName) = 0 Then Exit Function
   p = InStr(-1, strFileName, ".")
   If p Then strExt = Mid$(strFileName, p)
   If strExt = ".BAS" Or strExt = ".INC" Or strExt = ".RC" Then bValidExt = %TRUE
   Try
      nFile = FreeFile
      Open strFileName For Binary As nFile
      Get$ nFile, Lof(nFile), buffer   ' // Use GET$ because the file is ansi
   Catch
      MessageBox(0, "Error" & Str$(Err) & " loading the file " & strFileName, _
         FuncName$, %MB_Ok Or %MB_IconError Or %MB_TaskModal)
      Exit Function
   Finally
      Close nFile
   End Try

   ' // Check the first two bytes to see if the text contains the Unicode byte-order mark (BOM) &HFEFF.
   Local bIsUnicode As Long
   bIsUnicode = IsTextUnicode(StrPtr(buffer), 2, %IS_TEXT_UNICODE_SIGNATURE)
   If bIsUnicode Then buffer = ACode$(Mid$(buffer, 3))

   ' // Trim trailing spaces and tabs
   If pSed.TrimTrailingBlanks Then
      Do
         p = Len(buffer)
         Replace " " & $Cr With $Cr In buffer
         Replace $Tab & $Cr With $Cr In buffer
      Loop Until p = Len(buffer)
   End If

   ' // Get the file time
   fTime = CSED_GetFileTime(strFileName)

   ' // First look at already opened files
   hMdi = GetWindow(pSed.hwndClient, %GW_CHILD)
   While hMdi
      strPath = AfxGetWindowText(hMdi)
      ' // If already opened...
      If UCase$(strPath) = UCase$(strFileName) Then
         ' // Activate the MDI child window
         MdiActivate(pSed.hwndClient, hMdi)
         ' // Select the tab associated with the path
         nTab = CSED_GetTabNumberFromPath(strPath)
         If nTab Then TabCtrl_SetCurSel(pSed.hTabMdi, nTab)
         fModify = SCI_GetModify(pSed.hEdit)
         If fModify Then
            Beep
            r = CSED_Msgbox(pSed.hEdit, "Warning! The modifications will be lost." & $CrLf & _
               "Do you want to reload the file anyway?", %MB_YesNo Or %MB_IconWarning, "Reload file")
         End If
         If fModify And r = %IdYes Or fModify = %FALSE Then
            ' // Fill the control with the contents of the file
            SCI_SetText(pSed.hEdit, buffer)
            ' // Empty the undo buffer (it also sets the state of the document as unmodified)
            SCI_EmptyUndoBuffer(pSed.hEdit)
            ' // Store the time in the properties list of the window
            SetProp MdiGetActive(pSed.hwndClient), "FTIME", fTime
         End If
         ' // Load and set the bookmarks
         If pSed.SaveBookmarksToFile Then pSed.SetBookmarks(strFileName)
         ' // Update the MRU list and exit
         pSed.SetRecentFile(strFileName)
         CSED_RecentFilesMenu
         Exit Function
      End If
      hMdi = GetWindow(hMdi, %GW_HWNDNEXT)
   Wend

   ' // Create a new MDI child window
   hMdi = CSED_CreateMdiChild(strFileName)
   If hMdi Then
      ' // Fill the control with the contents of the file
      SCI_SetText(pSed.hEdit, buffer)
      ' // Empty the undo buffer (it also sets the state of the document as unmodified)
      SCI_EmptyUndoBuffer(pSed.hEdit)
      ' // Store the time in the properties list of the window
      SetProp MdiGetActive(pSed.hwndClient), "FTIME", fTime
      ' // Load and set the bookmarks
      If pSed.SaveBookmarksToFile Then pSed.SetBookmarks(strFileName)
      ' // Update the MRU list
      pSed.SetRecentFile(strFileName)
      CSED_RecentFilesMenu
   End If

   Function = hMdi

End Function
' ========================================================================================

' ========================================================================================
' Creates the code database
' ========================================================================================
Function CSED_CreateBackupMgrDatabase (ByVal hWnd As Dword, ByVal strDbPath As String) As Long

   Local strFileDef As String
   Local strKeys As String
   Local strSegment As String * 25

   strSegment = "Backup File Name"

   strFileDef = MkByt$(1) & _     ' // Page Size
                MkByt$(1) & _     ' // Compress Record
                MkByt$(1)         ' // Number of Key Segments

   strKeys =    strSegment  & _   ' // Description
                MkByt$(1)   & _   ' // Segment number
                MkWrd$(1)   & _   ' // Segment Offset
                MkByt$(60)  & _   ' // Segment Size
                MkByt$(2)         ' // Flags (No duplicates)

   strFileDef += strKeys

   ' // Create Tsunami database file
   If IsFile(strDbPath) Then
      If CSED_MsgBox(hWnd, strDbPath & $CrLf & "File already exist. Overwrite it?", %MB_Ok, "Warning!") = %IdYes Then
         Function = trm_Create(strDbPath, strFileDef, %TRUE)
      Else
         Exit Function
      End If
   Else
      Function = trm_Create(strDbPath, strFileDef, %FALSE)
   End If

End Function
' ========================================================================================

' ========================================================================================
' Save the record
' ========================================================================================
Function CSED_SaveBackupCodeRecord (ByVal hwnd As Dword, ByVal hBackupFile As Dword, ByVal strName As String, ByVal strText As String) As Long

   Local Result As Long
   Local RecordName As String * 60
   Local RecordStr As String

   RecordName = strName
   RecordStr = RecordName & strText
   ' // Insert the record
   Result = trm_Insert(hBackupFile, ByCopy RecordStr)
   ' // Duplicate key value: use trm_Update
   If Result = 5 Then
      ' // Establish current logical position
      trm_GetEqual(hBackupFile, 1, ByCopy RecordName)
      Result = trm_Result(hBackupFile)
      If Result = 0 Then Result = trm_Update(hBackupFile, ByCopy RecordStr)
   End If
   Function = Result

End Function
' ========================================================================================

' ========================================================================================
' Save file procedure
' ========================================================================================
#If %Def(%UNICODE)
Function CSED_SaveFile (ByVal hwnd As Dword, ByVal hSci As Dword, ByVal bAsk As Long) As Long
#Else
Function CSED_SaveFile (ByVal hwnd As Dword, ByVal hSci As Dword, ByVal bAsk As Long) As Long
#EndIf

   Local Result         As Long      ' // Result code
   Local dwStyle        As Dword     ' // Window styles
   Local nFile          As Dword     ' // File number
   Local buffer         As String    ' // Ansi string buffer
   Local bufLen         As Long      ' // Buffer length
   Local fTime          As Dword     ' // File time
   Local nTab           As Long      ' // Tab number
   Local hMdi           As Dword     ' // MDI child window
   Local hBackupFile    As Dword     ' // Backup file

#If %Def(%UNICODE)
   Local strInitialDir  As WString   ' // Initial directory
   Local strPath        As WString   ' // File path
   Local strExt         As WString   ' // File extension
   Local strOldExt      As WString   ' // Old file extension
   Local bakExt         As WString   ' // Backup file extension
   Local strDefExt      As WString   ' // DEfault file extension
   Local strFilter      As WString   ' // Filter
   Local strBakFileName As WString   ' // Backup file name
#Else
   Local strInitialDir  As String    ' // Initial directory
   Local strPath        As String    ' // File path
   Local strExt         As String    ' // File extension
   Local strOldExt      As String    ' // Old file extension
   Local bakExt         As String    ' // Backup file extension
   Local strDefExt      As String    ' // DEfault file extension
   Local strFilter      As String    ' // Filter
   Local strBakFileName As String    ' // Backup file name
#EndIf
   Local bstrPath       As WString   ' // Path in unicode
   Local strDbPath      As String    ' // Database path (ansi)
   Local strRecordName  As String    ' // Record Name

   strPath = AfxGetWindowText(GetParent(hSci))
   If IsNull(strPath) Then Exit Function
   ' // If not path, it's a new document
   If InStr(strPath, Any ":\/") = 0 Then
      strInitialDir = CurDir$
      If Right$(strInitialDir, 1) <> "\" Then strInitialDir = strInitialDir + "\"
      If Left$(UCase$(strPath), 8) = "UNTITLED" And InStr(strPath, ".") = 0 Then strPath += ".BAS"
      ' // We need the dialog for new docs
      bAsk = %TRUE
   Else
      strInitialDir = PathName$(Path, strPath)
      strPath = PathName$(Namex, strPath)
   End If
   strDefExt = "BAS"
   If IsFile(ProjectFile) Then strInitialDir = PathName$(Path,ProjectFile)

   ' // Build the filter based on the file extension
   strExt = UCase$(PathName$(Extn, strPath))
   strOldExt = strExt
   Select Case strExt
      Case ".BAS"
         strFilter += "PB Code Files (*.BAS)|*.BAS|"
         strFilter += "PB Include Files (*.INC)|*.INC|"
         strDefExt = "BAS"
      Case ".INC"
         strFilter += "PB Include Files (*.INC)|*.INC|"
         strFilter += "PB Code Files (*.BAS)|*.BAS|"
         strDefExt = "INC"
      Case ".PBTPL"
         strFilter += "PB Template Files (*.PBTPL)|*.PBTPL|"
         strDefExt = "PBTPL"
      Case ".RC", ".RC2", ".DLG"
         strFilter += "Resource Files (*.RC;*.RC2;.DLG)|*.RC;*.RC2;*.DLG|"
         strDefExt = "RC"
      Case ".TXT"
         strFilter += "Text Files (*.TXT)|*.TXT|"
         strDefExt = "TXT"
      Case ".XML", ".XSL", ".XSD"
         strFilter += "XML Files (*.XML;*.XSL;*.XSD)|*.XML;*.XSL;*.XSD|"
         strDefExt = "XML"
      Case ".HTML", ".HTM", ".SHTML"
         strFilter += "HTML Files (*.HTML;*.HTM;*.SHTML)|*.HTML;*.HTM;*.SHTML|"
         strDefExt = "HTML"
      Case ".C", ".CC", ".CPP", ".CXX", ".H", ".HH", ".HPP", ".HXX"
         strFilter += "C/CPP Files (*.C;*.CC;*.CPP;*.CXX;*.H;*.HH;*.HPP;*.HXX)|*.C;*.CC;*.CPP;*.CXX;*.H;*.HH;*.HPP;*.HXX|"
         strDefExt = "C"
      Case ".JAVA", ".JAD"
         strFilter += "Java Files (*.JAVA;*.JAD)|*.JAVA;*.JAD|"
         strDefExt = "JAVA"
      Case ".JS"
         strFilter += "Java Script Files (*.JS)|*.JS|"
         strDefExt = "JS"
      Case ".IDL", ".ODL"
         strFilter += "IDL Files (*.IDL;*.ODL)|*.IDL;*.ODL|"
         strDefExt = "IDL"
   End Select
   strFilter += "All Files (*.*)|*.*"

   ' // Launch the Save File Dialog
   If bAsk Then
      dwStyle = %OFN_Explorer Or %OFN_FileMustExist Or %OFN_HideReadOnly Or %OFN_OverWritePrompt
      If IsFalse(AfxSaveFileDialog(pSed.hwndMain, "", strPath, strInitialDir, strFilter, strDefExt, dwStyle)) Then Exit Function
   Else
      strPath = strInitialDir & strPath
   End If

   ' // Backup the file
   If pSed.BackupEditorFiles Then
      strExt = UCase$(PathName$(Extn, strPath))
      If strExt <> ".BAK" Then
         strBakFileName = strPath
         strBakFileName += ".bak"
         bakExt = strExt & ".bak"
         If pSed.AddDateTimeToBackupFiles Then
            Local d, t, dt As String
            d = Date$
            d = Right$(d, 4) & Left$(d, 2) & Mid$(d, 4, 2)
            t = Time$
            t = Remove$(t, ":")
            dt = d & "_" & t
          strBakFileName = Left$(strBakFilename, Len(strBakFilename) - Len(bakExt)) & "_" & dt & bakExt
         End If
         If IsFalse pSed.SaveBackupFilesToDatabase Then
            If IsFile(strPath) Then FileCopy strPath, strBakFileName
         End If
      End If
   End If

   ' // Save the file
   Try
      nFile = FreeFile
      Open strPath For Binary As nFile
      ' // Get the text to save
      buffer = SCI_GetText(hSci)
      ' // Remove trailing spaces and tabs
      If pSed.TrimTrailingBlanks Then
         Do
            bufLen = Len(buffer)
            Replace " " & $Cr With $Cr In buffer
            Replace $Tab & $Cr With $Cr In buffer
         Loop Until bufLen = Len(buffer)
      End If
      Put$ nFile, buffer
      SetEof nFile
   Catch
      CSED_MsgBox(hwnd, "Error" & Str$(Err) & " saving the file " & strPath, _
         %MB_Ok Or %MB_IconError, "Save File")
      Exit Function
   Finally
      Close nFile
   End Try

   ' // Tell to Scintilla that the current state of the document is unmodified
   SCI_SetSavePoint(hSci)

   ' // Save backup file in the database
   If pSed.BackupEditorFiles And pSed.SaveBackupFilesToDatabase Then
      strDbPath = Exe.Path$ & "CSED_BACKUPMGR\" & $BACKUPSDB
      If IsFalse IsFile(strDbPath) Then
         hBackupFile = CSED_CreateBackupMgrDatabase(hwnd, strDbPath)
      End If
      If hBackupFile < 1 Or IsFalse trm_FileIsOpen(hBackupFile) Then
         hBackupFile = trm_Open(strDbPath, %TRUE)
         If hBackupFile < 1 Then
            CSED_MsgBox(hwnd, "Error opening the file backups database", _
               %MB_Ok Or %MB_IconError, "Tsunami Error")
         Else
            strRecordName = PathName$(Name, strBakFileName)
            Result = CSED_SaveBackupCodeRecord(hwnd, hBackupFile, strRecordName, buffer)
            If Result Then
               CSED_MsgBox(hwnd, "Error " & Format$(Result) & " saving the file backup record", _
                  %MB_Ok Or %MB_IconError, "Tsunami Error")
            End If
            trm_Close hBackupFile
         End If
      End If
   End If

   ' // Save the bookmarks
   If pSed.SaveBookmarksToFile And _
      LCase$(strExt) <> ".csbkm" And _
      LCase$(strExt) <> ".bak" Then
      pSed.SaveBookmarks(hSci, strPath)
   End If

   ' // Update reopen file list (MRU menu)
   pSed.SetRecentFile(strPath)
   CSED_RecentFilesMenu

   ' // Set Scintilla options if the user has changed the file extension
   If UCase$(PathName$(Extn, strPath)) <> strOldExt Then
      strExt = UCase$(PathName$(Extn, strPath))
      CSED_SetScintillaOptions hSci, strExt
   End If

   ' // Get the handle of the MDI child window
   hMdi = GetParent(hSci)

   If bAsk Then
      ' // Get the current tab
      nTab = TabCtrl_GetCurSel(pSed.hTabMdi)
      ' // Update the collection of full paths
      ' // --------------------------------------------
      ' // Workaround: The Add method of the linked list
      ' // collection needs an Unicode string
      bStrPath = strPath
      pSed.TabFilePaths.Replace(nTab + 1, bstrPath)
      ' // --------------------------------------------
      ' // Update the window caption and the tab name
      If hMdi Then AfxSetWindowText hMdi, strPath
      TabCtrl_SetText(pSed.hTabMdi, nTab, PathName$(Namex, strPath))
   End If

   ' // Retrieve the file time and store it in the properties list of the window
   fTime = CSED_GetFileTime(strPath)
   If hMdi Then SetProp hMdi, "FTIME", fTime

   Function = -1

End Function
' ========================================================================================

' ========================================================================================
' Save untitled file in Windows temporary folder
' ========================================================================================
#If %Def(%UNICODE)
Function CSED_SaveUntitledFile (ByVal hwnd As Dword, ByVal hSci As Dword, ByVal strPath As WString) As Long
#Else
Function CSED_SaveUntitledFile (ByVal hwnd As Dword, ByVal hSci As Dword, ByVal strPath As String) As Long
#EndIf

   Local nLen   As Long                 ' // Text lenght
   Local nFile  As Long                 ' // File number
   Local buffer As String               ' // Buffer
#If %Def(%UNICODE)
   Local szTempPath As WStringZ * %Max_Path   ' // Temporary file path
#Else
   Local szTempPath As AsciiZ * %Max_Path     ' // Temporary file path
#EndIf

   If IsNull(strPath) Then Exit Function
   GetTempPath(%Max_Path, szTempPath)
   ChDir szTempPath
   strPath = szTempPath & strPath
   nFile = FreeFile
   Open strPath For Binary As nFile
   If Err Then
      CSED_MsgBox(hwnd, "Error" & Str$(Err) & " saving the file", _
         %MB_Ok Or %MB_IconError Or %MB_ApplModal, "Save untitled file")
      Exit Function
   End If
   nLen = SCI_GetTextLength(hSci)
   buffer = SCI_GetText(hSci)
   Put$ nFile, Left$(buffer, Len(buffer))
   SetEof nFile
   Close nFile
   Function = %TRUE

End Function
' ========================================================================================

' ========================================================================================
' Insert file procedure
' ========================================================================================
Sub CSED_InsertFile (ByVal hwnd As Dword)

#If %Def(%UNICODE)
   Local strPath        As WString   ' // File path
   Local strInitialDir  As WString   ' // Initial directory
   Local strFilter      As WString   ' // Filter
#Else
   Local strPath        As String    ' // File path
   Local strInitialDir  As String    ' // Initial directory
   Local strFilter      As String    ' // Filter
#EndIf
   Local dwStyle        As Dword     ' // Window styles
   Local nFile          As Long      ' // File number
   Local buffer         As String    ' // Buffer

   If IsFalse pSed.hEdit Then Exit Sub

   strInitialDir = CurDir$
   strFilter = "PB Code Files (*.BAS)|*.BAS|"
   strFilter += "PB Include Files (*.INC)|*.INC|"
   strFilter += "PB Include Files (*.INC)|*.INC|"
   strFilter += "PB Code Files (*.BAS)|*.BAS|"
   strFilter += "All Files (*.*)|*.*"
   strPath = "*.BAS;*.INC"

   ' // Launch the Open File Dialog
   dwStyle = %OFN_Explorer Or %OFN_FileMustExist Or %OFN_HideReadOnly Or %OFN_LongNames
   If IsFalse(AfxOpenFileDialog(pSed.hwndMain, "", strPath, strInitialDir, strFilter, "BAS", dwStyle)) Then Exit Sub

   ' // Open the file and read it
   Try
      nFile = FreeFile
      Open strPath For Binary As nFile
      Get$ nFile, Lof(nFile), Buffer
   Catch
      CSED_MsgBox(hwnd, "Error" & Str$(Err) & " inserting the file " & strPath, _
         %MB_Ok Or %MB_IconError, "Insert File")
      Exit Sub
   Finally
      Close nFile
   End Try

   ' // Insert the text at the current position
   SCI_InsertText(pSed.hEdit, -1, buffer)
   ' // Change statusbar information
   CSED_ShowLinCol

End Sub
' ========================================================================================

' ========================================================================================
' GetDroppedFiles - Retrieves the paths of the dropped files
' ========================================================================================
Function CSED_GetDroppedFiles (ByVal hDrop As Dword) As IQueueCollection

   Local i      As Long                   ' // Loop counter
   Local nCount As Long                   ' // Number of dropped filed
   Local nLen   As Long                   ' // Length of the path + filename
   Local pQueue As IQueueCollection       ' // Reference to the Queue collection
#If %Def(%UNICODE)
   Local szPath As WStringZ * %Max_Path   ' // Filename and path
#Else
   Local szPath As AsciiZ * %Max_Path     ' // Filename and path
#EndIf

   ' // Get the number of dropped files
   nCount = DragQueryFile(hDrop, &HFFFFFFFF???, ByVal %NULL, 0)
   If nCount = 0 Then Exit Function

   ' // Create an instance of the Queue collection
   pQueue = Class "QueueCollection"
   If IsNothing(pQueue) Then Exit Function

   For i = 0 To nCount - 1
      ' // Retrieve the path and length
      nLen = DragQueryFile(hDrop, i, szPath, %Max_Path)
      If nLen Then
         szPath = Trim$(Left$(szPath, nLen))
         ' // Make sure it's a file, not a folder
         If Len(szPath) And (GetAttr(szPath) And 16) = 0 Then
            pQueue.Enqueue ByCopy szPath
         End If
      End If
   Next

   DragFinish hDrop
   Function = pQueue

End Function
' ========================================================================================

' ========================================================================================
' Load selected file
' ========================================================================================
Sub CSED_LoadSelectedFile (ByVal hwnd As Dword, ByVal strFileName As String)

   Local i          As Long              ' // Loop counter
   Local RetVal     As Long              ' // Return value
   Local hFile      As Dword             ' // File handle
   Local strPath    As String            ' // File path
   Local strIncPath As String            ' // Include files path
   Local WFD        As WIN32_FIND_DATA   ' // Find file structure

   If InStr(strFileName, Any ":\/") Then
      hFile = FindFirstFile(ByCopy strFileName, WFD)
      If hFile = %INVALID_HANDLE_VALUE Then
         CSED_MsgBox(0, strFileName & " not found", %MB_Ok, "File not found")
         Exit Sub
      End If
      strPath = strFileName
   Else
      ' // File has not path; add the path of the source file
      strPath = AfxGetWindowText(hwnd)
      strPath = PathName$(Path, strPath)
      strPath += strFileName
      hFile = FindFirstFile(ByCopy strPath, WFD)
      If hFile = %INVALID_HANDLE_VALUE Then
         ' // Search the file in the include folders
         If pSed.SelectedHeaders = 1 Or pSed.SelectedHeaders = 0 Then
            strIncPath = pSed.JRSIncPath1
         ElseIf pSed.SelectedHeaders = 2 Then
            strIncPath = pSed.JRSIncPath2
         ElseIf pSed.SelectedHeaders = 3 Then
            strIncPath = pSed.PBIncPath1
         ElseIf pSed.SelectedHeaders = 4 Then
            strIncPath = pSed.PBIncPath2
         End If
         RetVal = ParseCount(strIncPath , ";")
         For i = 1 To RetVal
            strPath = Parse$(strIncPath , ";" , i) & "\" & strFileName
            hFile = FindFirstFile(ByCopy strPath, WFD)
            If hFile <> %INVALID_HANDLE_VALUE Then Exit For
         Next
         If hFile = %INVALID_HANDLE_VALUE Then
            CSED_MsgBox(0, strPath & " not found", %MB_Ok, "File not found")
            Exit Sub
         End If
      End If
   End If
   ' // Close the search handle
   FindClose hFile
   ' // Open the file
   CSED_OpenFile strPath

End Sub
' ========================================================================================

' ========================================================================================
' Inserts the text of the macro at the caret position
' ========================================================================================
Sub CSED_InsertMacroText (ByVal hwnd As Dword, ByVal wParam As Dword)

   Local hFile     As Long
   Local strDbPath As String
   Local RecordStr As String
   Local strKey    As String
   Local wKey      As Word

   wKey = wParam
   Select Case wKey
      Case 3700 To 3709
         strKey = "Alt" & Format$(wKey - 3700)
      Case 3710 To 3719
         strKey = "Ctrl" & Format$(wKey - 3710)
   End Select

   strDbPath = Exe.Path$ & "CSED_KBDMACROS\CSED_KBDMACROS.TRM"
   If IsFalse IsFile(strDbPath) Then
      CSED_MsgBox(hwnd, "Can't find the file " & strDbPath, %MB_Ok Or %MB_IconError, "CSED Macros")
      Exit Sub
   End If

   hFile = trm_Open(strDbPath, %TRUE)
   If hFile < 1 Then
      CSED_MsgBox(hwnd, "Error opening  " & strDbPath, %MB_Ok Or %MB_IconError, "CSED Macros")
      Exit Sub
   End If

   RecordStr = trm_GetEqualOrGreater(hFile, 1, ByCopy strKey)
   If UCase$(strKey) = Trim$(UCase$(Left$(RecordStr, 15))) Then
      RecordStr = Mid$(RecordStr, 76)
      SCI_InsertText(pSed.hEdit, -1, RecordStr)
   End If

   trm_Close(hFile)

End Sub
' ========================================================================================

' ========================================================================================
' Registers file extensions
' ========================================================================================
Sub CSED_RegisterFileExtensions (ByVal hwnd As Dword)

   Local r          As Long
   Local KeyHandle  As Dword
#If %Def(%UNICODE)
   Local KeyName    As WStringZ * 256
   Local KeyValue   As WStringZ * 256
#Else
   Local KeyName    As AsciiZ * 256
   Local KeyValue   As AsciiZ * 256
#EndIf

   r = CSED_MsgBox(hwnd, "This will associate the .BAS, .INC .SED and .RC file extensions with the editor." & $CrLf & _
                         "You must have started the editor as administrator to succeed.", %MB_OkCancel, "Register File Extensions")
   If r = %IdCancel Then Exit Sub

   KeyName = ".BAS"
   KeyValue = "CSED Editor"
   r = RegCreateKey(%HKEY_CLASSES_ROOT, KeyName, KeyHandle)
   If r <> %ERROR_SUCCESS Then MsgBox AfxWinErrMsg(r) : Exit Sub
   r = RegSetValue(KeyHandle, "", %REG_SZ, KeyValue, 0)
   RegCloseKey KeyHandle
   If r <> %ERROR_SUCCESS Then MsgBox AfxWinErrMsg(r) : Exit Sub

   KeyName = ".INC"
   KeyValue = "CSED Editor"
   r = RegCreateKey(%HKEY_CLASSES_ROOT, KeyName, KeyHandle)
   If r <> %ERROR_SUCCESS Then MsgBox AfxWinErrMsg(r) : Exit Sub
   r = RegSetValue(KeyHandle, "", %REG_SZ, KeyValue, 0)
   RegCloseKey KeyHandle
   If r <> %ERROR_SUCCESS Then MsgBox AfxWinErrMsg(r) : Exit Sub

   KeyName = ".SED"
   KeyValue = "CSED Editor"
   r = RegCreateKey(%HKEY_CLASSES_ROOT, KeyName, KeyHandle)
   If r <> %ERROR_SUCCESS Then MsgBox AfxWinErrMsg(r) : Exit Sub
   r = RegSetValue(KeyHandle, "", %REG_SZ, KeyValue, 0)
   RegCloseKey KeyHandle
   If r <> %ERROR_SUCCESS Then MsgBox AfxWinErrMsg(r) : Exit Sub

   KeyName = ".RC"
   KeyValue = "CSED Editor"
   r = RegCreateKey(%HKEY_CLASSES_ROOT, KeyName, KeyHandle)
   If r <> %ERROR_SUCCESS Then MsgBox AfxWinErrMsg(r) : Exit Sub
   r = RegSetValue(KeyHandle, "", %REG_SZ, KeyValue, 0)
   RegCloseKey KeyHandle
   If r <> %ERROR_SUCCESS Then MsgBox AfxWinErrMsg(r) : Exit Sub

   KeyName = "CSED Editor"
   KeyValue = "CSED File Extensions"
   r = RegCreateKey(%HKEY_CLASSES_ROOT, KeyName, KeyHandle)
   If r <> %ERROR_SUCCESS Then MsgBox AfxWinErrMsg(r) : Exit Sub
   r = RegSetValue(KeyHandle, "", %REG_SZ, KeyValue, 0)
   RegCloseKey KeyHandle
   If r <> %ERROR_SUCCESS Then MsgBox AfxWinErrMsg(r) : Exit Sub

   KeyName = "CSED Editor"
   KeyValue = Exe.Path$ & "\Resources\CSED.ICO"
   r = RegCreateKey(%HKEY_CLASSES_ROOT, KeyName, KeyHandle)
   If r <> %ERROR_SUCCESS Then MsgBox AfxWinErrMsg(r) : Exit Sub
   r = RegSetValue(KeyHandle, "ICON", %REG_SZ, KeyValue, %Max_Path)
   RegCloseKey KeyHandle
   If r <> %ERROR_SUCCESS Then MsgBox AfxWinErrMsg(r) : Exit Sub

   KeyName = "CSED Editor"
   KeyValue = Exe.Full$ & " %1"
   r = RegCreateKey(%HKEY_CLASSES_ROOT, KeyName, KeyHandle)
   If r <> %ERROR_SUCCESS Then MsgBox AfxWinErrMsg(r) : Exit Sub
   r = RegSetValue(KeyHandle, "shell\open\command", %REG_SZ, KeyValue, %Max_Path)
   RegCloseKey KeyHandle
   If r <> %ERROR_SUCCESS Then MsgBox AfxWinErrMsg(r) : Exit Sub

   CSED_MsgBox hwnd, ".BAS, .INC .SED and .RC file extensions" & $CrLf & "have been registered with Windows", %MB_Ok, "Register File Extensions"

End Sub
' ========================================================================================
